"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Environment = exports.BString = exports.BArray = exports.Char = exports.Int = exports.Field = void 0;
class Field {
    constructor(environment) {
        this.environment = environment;
        this.index = this.environment.allocateIndex();
    }
    focus() {
        this.environment.focusIndex(this.index);
        return this;
    }
    execute(code, comment) {
        this.focus();
        this.environment.execute(code, comment);
        return this;
    }
    reset() {
        return this.execute("[-]", "Reset field to 0");
    }
    set(value) {
        this.reset();
        if (value > 0)
            this.execute("+".repeat(value), "Set to " + value);
        else if (value < 0)
            this.execute("-".repeat(value), "Set to " + value);
        return this;
    }
    setInt(int) {
        return this.set(int);
    }
    setChar(char) {
        return this.set(char.charCodeAt(0));
    }
    setBool(bool) {
        return this.setInt(bool ? 1 : 0);
    }
    input() {
        return this.execute(",");
    }
    move(...others) {
        for (const other of others) {
            other.reset();
        }
        this.environment.executeWhile(this, () => {
            for (const other of others) {
                other.execute("+");
            }
            this.execute("-");
        });
        return this;
    }
    copy(...other) {
        const intermediate = new Field(this.environment);
        this.move(intermediate);
        intermediate.move(this, ...other);
        return this;
    }
    print() {
        return this.execute(".");
    }
    clone() {
        const n = new this.constructor(this.environment);
        this.copy(n);
        return n;
    }
}
exports.Field = Field;
class Int extends Field {
    subtract(other) {
        const intermediate = new Field(this.environment);
        other.copy(intermediate);
        intermediate.execute("[");
        this.execute("-");
        intermediate.execute("-]");
        return this;
    }
    add(other) {
        const intermediate = new Field(this.environment);
        other.copy(intermediate);
        intermediate.execute("[");
        this.execute("+");
        intermediate.execute("-]");
        return this;
    }
    multiply(other) {
        const intermediateMultiplier = other.clone();
        const intermediateThis = this.clone();
        this.reset();
        this.environment.executeWhile(intermediateMultiplier, () => {
            this.add(intermediateThis);
            intermediateMultiplier.execute("-");
        });
        return this;
    }
}
exports.Int = Int;
class Char extends Field {
}
exports.Char = Char;
class BArray {
    constructor(environment, items) {
        this.environment = environment;
        this.items = items;
        this.items = items;
    }
    print() {
        new Char(this.environment).setChar("[").print();
        for (const item of this.items) {
            item.print();
            new Char(this.environment).setChar(",").print();
        }
        new Char(this.environment).setChar("]").print();
    }
}
exports.BArray = BArray;
class BString extends BArray {
    constructor(environment, string) {
        super(environment, string.split("").map((char) => new Char(environment).setChar(char)));
        this.environment = environment;
    }
    input() {
        for (const item of this.items) {
            item.input();
        }
    }
    print() {
        for (const item of this.items) {
            item.print();
        }
        // const goon = Field.fromBool(true)
        // for(const item of this.items) {
        //     addIf(goon, () => {
        //         goon.setBool(false)
        //         addIf(item, () => {
        //             item.print()
        //             goon.setBool(true)
        //         })
        //     })
        // }
    }
}
exports.BString = BString;
class Environment {
    constructor() {
        this.code = "";
        this.usedIndexes = 0;
        this.currentIndex = 0;
    }
    execute(code, comment) {
        this.code += code;
    }
    executeIf(field, callback) {
        const falsy = new Field(this).setBool(false);
        field.execute("[");
        callback();
        falsy.execute("]");
    }
    executeWhile(field, callback) {
        field.execute("[");
        callback();
        field.execute("]");
    }
    allocateIndex() {
        return this.usedIndexes++;
    }
    focusIndex(index) {
        const diff = index - this.currentIndex;
        if (diff > 0)
            this.execute(">".repeat(diff), "goto " + index);
        else if (diff < 0)
            this.execute("<".repeat(-diff), "goto " + index);
        this.currentIndex = index;
    }
}
exports.Environment = Environment;
